/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/

// HOExtrapBC.H

//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// Dan Martin, Fri, Jan 14, 2000

#ifndef _HOExtrapBC_H_
#define _HOExtrapBC_H_

#include "GhostBC.H"


/// class to do extrapolation BC's 
class HOExtrapBC : public BoxGhostBC
{
public:
  ///
  friend class DomainGhostBC;

  /// 
  HOExtrapBC();

  ///
  ~HOExtrapBC();

  /// 
  HOExtrapBC(int dir, Side::LoHiSide sd, 
	     const Interval& a_comps);

  ///
  HOExtrapBC(int dir, Side::LoHiSide sd);
	     
  
protected:
  
  ///
  virtual void applyHomogeneousBCs(FArrayBox& a_state,
				   const ProblemDomain& a_domain,
				   Real a_dx) const;
  
  ///
  virtual void applyInhomogeneousBCs(FArrayBox& a_state,
				     const ProblemDomain& a_domain,
				     Real a_dx) const;
  
  ///
  virtual void applyHOExtrapBCs(FArrayBox& a_state, 
			      const ProblemDomain& a_domain,
			      Real a_dx) const;
  

  /// this doesn't do anything since we don't use the factors
  virtual void fillBCValues(FArrayBox& a_neumfac,
			    FArrayBox& a_dircfac,
			    FArrayBox& a_inhmval,
			    Real a_dx, 
			    const ProblemDomain& a_domain) const;


  /// deprecated interface
  virtual void applyHomogeneousBCs(FArrayBox& a_state,
				   const Box& a_domain,
				   Real a_dx) const;
  
  /// deprecated interface
  virtual void applyInhomogeneousBCs(FArrayBox& a_state,
				     const Box& a_domain,
				     Real a_dx) const;
  
  /// deprecated interface
  virtual void applyHOExtrapBCs(FArrayBox& a_state, 
			      const Box& a_domain,
			      Real a_dx) const;
  

  /// this doesn't do anything since we don't use the factors
  virtual void fillBCValues(FArrayBox& a_neumfac,
			    FArrayBox& a_dircfac,
			    FArrayBox& a_inhmval,
			    Real a_dx, 
			    const Box& a_domain) const;
  
  /// 
  BoxGhostBC* new_boxghostbc() const;
  
private:
  HOExtrapBC(const HOExtrapBC& extrapin) {;}
  void operator=(const HOExtrapBC& extrapin) {;}
  
};




#endif


