/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/

// viscousBC.cpp

//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// Dan Martin, Fri, Jan 14, 2000

#include "viscousBC.H"
#include "viscousBCF_F.H"

viscousBC::viscousBC()
{
}

viscousBC::~viscousBC()
{
}

void
viscousBC::applyBCs(LevelData<FArrayBox>& a_viscousTerm,
		    const Box& a_domain, const Real a_dx) const
{
  ProblemDomain physdomain(a_domain);
  applyBCs(a_viscousTerm, physdomain, a_dx);
}


void
viscousBC::applyBCs(LevelData<FArrayBox>& a_viscousTerm,
		    const ProblemDomain& a_domain, const Real a_dx) const
{

  const DisjointBoxLayout& grids = a_viscousTerm.getBoxes();

  DataIterator dit = a_viscousTerm.dataIterator();

  for (dit.begin(); dit.ok(); ++dit) {
    applyBCs(a_viscousTerm[dit()], grids[dit()], a_dx);
  } // end loop over boxes
}


void
viscousBC::applyBCs(FArrayBox& a_state, const Box& a_domain,
		    const Real a_dx) const
{    
  ProblemDomain physdomain(a_domain);
  applyBCs(a_state, physdomain, a_dx);
}


void
viscousBC::applyBCs(FArrayBox& a_state, const ProblemDomain& a_domain,
		    const Real a_dx) const
{    
  
  int nComp = a_state.nComp();
  
  // now loop over directions
  for (int dir=0; dir<SpaceDim; dir++) {
    // only do anything in non-periodic case
    if (!a_domain.isPeriodic(dir))
    {
      const Box& thisBox = a_domain.domainBox();
      const Box& grownBox = a_state.box();
      Box loBox;
      Box hiBox;
      
      if (grownBox.smallEnd(dir) < thisBox.smallEnd(dir)) {
        loBox = grownBox;
        loBox.setBig(dir,thisBox.smallEnd(dir)-1);
      }
      if (grownBox.bigEnd(dir) > thisBox.bigEnd(dir)) {
        hiBox = grownBox;
        hiBox.setSmall(dir,thisBox.bigEnd(dir)+1);
      }
    
    
      FORT_VISCOUSBC(CHF_FRA(a_state),
                     CHF_BOX(thisBox),
                     CHF_BOX(loBox),
                     CHF_BOX(hiBox),
                     CHF_INT(dir),		       
                     CHF_INT(nComp));

      
    } // end if not periodic in this direction
  } // end loop over directions
  
} 


void 
viscousBC::applyBCs(FArrayBox& a_state, const Box& a_domain,
		    const Real a_dx, int a_comp) const
{

  // haven't implemented this yet -- will do it if necessary
  MayDay::Error("viscousBC::applyBC for single component not implemented!");
}



void 
viscousBC::applyBCs(FArrayBox& a_state, const ProblemDomain& a_domain,
		    const Real a_dx, int a_comp) const
{

  // haven't implemented this yet -- will do it if necessary
  MayDay::Error("viscousBC::applyBC for single component not implemented!");
}




			   

