/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/

// GodunovTrace.H

//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// Dan Martin, Fri, Jan 14, 2000

#ifndef _GodunovTrace_H_
#define _GodunovTrace_H_

#include "FluxBox.H"
#include "FArrayBox.H"
#include "ProblemDomain.H"


// Set of Functions which perform upwinded tracing 


/*@ManMemo: trace state to edges in direction dir at time t + dt/2
  -- this version uses edge-centered advection velocities which
  are passed in */
void TraceState(/// state at time t+dt/2 on edges in direction dir
		FArrayBox& a_stateHalf, 
		/// cell-centered state at time t
		const FArrayBox& a_state,
		/// cell-centered velocity at time t
		const FArrayBox& a_cellVel,
		/// edge-centered advection velocity at time t+dt/2
		const FluxBox& a_advectionVel,
		/// cell-centered source
		const FArrayBox& a_source,
		/// Physical domain
		const ProblemDomain& a_dProblem,
		/// interior of grid patch
		const Box& a_gridBox,
		/// timeStep
		const Real a_dt,
		/// cell-spacing
		const Real a_dx,
		/// direction in which to perform tracing
		const int dir,
		/// which components to trace
		const Interval& a_srcComps,
		///  where to put traced components in a_stateHalf,
		const Interval& a_destComps);


/*@ManMemo: trace state to edges in direction dir at time t + dt/2
  -- this version uses edge-centered advection velocities which
  are passed in; deprecated interface using a Box for the domain
  instead of a ProblemDomain (will default to non-periodic case) */
void TraceState(/// state at time t+dt/2 on edges in direction dir
		FArrayBox& a_stateHalf, 
		/// cell-centered state at time t
		const FArrayBox& a_state,
		/// cell-centered velocity at time t
		const FArrayBox& a_cellVel,
		/// edge-centered advection velocity at time t+dt/2
		const FluxBox& a_advectionVel,
		/// cell-centered source
		const FArrayBox& a_source,
		/// Physical domain
		const Box& a_dProblem,
		/// interior of grid patch
		const Box& a_gridBox,
		/// timeStep
		const Real a_dt,
		/// cell-spacing
		const Real a_dx,
		/// direction in which to perform tracing
		const int dir,
		/// which components to trace
		const Interval& a_srcComps,
		///  where to put traced components in a_stateHalf,
		const Interval& a_destComps);


/*@ManMemo: trace state to edges in direction dir at time t+dt/2
  -- this version does not have edge-centered advection velocities
  passed in; uses cell-centered velocity field to approximate them */
void TraceState(/// state at time t+dt/2 on edges in direction dir
		FArrayBox& a_stateHalf, 
		/// cell-centered state at time t
		const FArrayBox& a_state,
		/// cell-centered velocity at time t
		const FArrayBox& a_cellVel,
		/// cell-centered source
		const FArrayBox& a_source,
		/// Physical domain
		const ProblemDomain& a_dProblem,
		/// interior of grid patch
		const Box& a_gridBox,
		/// timeStep
		const Real a_dt,
		/// cell-spacing
		const Real a_dx,
		/// direction in which to perform tracing
		const int dir,
		/// which components to trace
		const Interval& a_srcComps,
		///  where to put traced components in a_stateHalf,
		const Interval& a_destComps);


/*@ManMemo: trace state to edges in direction dir at time t+dt/2
  -- this version does not have edge-centered advection velocities
  passed in; uses cell-centered velocity field to approximate them.
  deprecated interface using a Box for the domain
  instead of a ProblemDomain (will default to non-periodic case) */

void TraceState(/// state at time t+dt/2 on edges in direction dir
		FArrayBox& a_stateHalf, 
		/// cell-centered state at time t
		const FArrayBox& a_state,
		/// cell-centered velocity at time t
		const FArrayBox& a_cellVel,
		/// cell-centered source
		const FArrayBox& a_source,
		/// Physical domain
		const Box& a_dProblem,
		/// interior of grid patch
		const Box& a_gridBox,
		/// timeStep
		const Real a_dt,
		/// cell-spacing
		const Real a_dx,
		/// direction in which to perform tracing
		const int dir,
		/// which components to trace
		const Interval& a_srcComps,
		///  where to put traced components in a_stateHalf,
		const Interval& a_destComps);
		
		
		
		
		


#endif


