/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/

// CoarseAverageEdge.H

//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// Dan Martin, Fri, Jan 14, 2000

#ifndef _CoarseAverageEdge_H_
#define _CoarseAverageEdge_H_

#include "REAL.H"
#include "FluxBox.H"
#include "LevelData.H"


/// replaces edge-centered coarse-level data w/ averaged fine-level data
/** This class replaces edge-centered data on a coarse level of refinement
    with the average of the finer-level data which overlays the edge.
    This class is similar to CoarseAverage */
class CoarseAverageEdge
{
public:
  /// Default constructor
  CoarseAverageEdge();

  /// defining constructor
  CoarseAverageEdge(const DisjointBoxLayout& a_fineGrids, 
		    int a_nComp, int a_nRef);

  /// destructor
  ~CoarseAverageEdge();

  /// defines the object
  void define(const DisjointBoxLayout& a_fineGrids, 
	      int a_nComp, int a_nRef);
  
  ///
  bool isDefined() const;

  /// averages fine-level data to coarse level
  void averageToCoarse(LevelData<FluxBox>& a_coarse_data, 
		       const LevelData<FluxBox>& a_fine_data);

protected:
  bool m_isDefined;

  int m_nRef;

  // work array for coarsening of fine data, same "shape" as fine data
  LevelData<FluxBox> m_coarsenedFineData;

  /// averages entire single grid data from fine->crse
  void averageGridData(FluxBox& a_coarsenedFine, const FluxBox& fine) const;

  
};

#endif
  



