/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/

// EdgeToCell.cpp

//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// Dan Martin, Fri, Jan 14, 2000

#include <assert.h>
#include "DataIterator.H"
#include "EdgeToCell.H"
#include "EdgeToCellF_F.H"

// ----------------------------------------------------------------
void EdgeToCell(const LevelData<FluxBox>& a_edgeData,
		LevelData<FArrayBox>& a_cellData)

{
  // this is just a wrapper around the single-grid version
  DataIterator dit = a_edgeData.dataIterator();
  for (dit.reset(); dit.ok(); ++dit) 
    {
      EdgeToCell(a_edgeData[dit()], a_cellData[dit()]);
    }
}


// ----------------------------------------------------------------
void EdgeToCell(const FluxBox& a_edgeData,
		FArrayBox& a_cellData)
{

  // quick check to make sure this will work
  //assert(a_edgeData.nComp() == a_cellData.nComp());

  // loop over componentns -- assumption is that in cell-centered
  // data, direction changes faster than component.
  for (int comp = 0; comp<a_edgeData.nComp(); comp++) 
    {
      // loop over directions
      for (int dir = 0; dir < SpaceDim; dir++) 
	{
	  Box cellBox = a_cellData.box();
	  cellBox &= a_edgeData.box();
	  int cellcomp = SpaceDim*comp + dir;
	  FORT_EDGETOCELL(CHF_CONST_FRA1(a_edgeData[dir],comp),
			  CHF_FRA1(a_cellData, cellcomp),
			  CHF_BOX(cellBox),
			  CHF_CONST_INT(dir));
	} // end loop over directions
    } // end loop over components
}


void EdgeToCell(const FluxBox& a_edgeData, const int a_edgeComp,
		FArrayBox& a_cellData, const int a_cellComp, const int a_dir)
{

  // quick check to make sure this will work
  //assert(a_edgeData.nComp() == a_cellData.nComp());

  const Box& cellBox = a_cellData.box();
  FORT_EDGETOCELL(CHF_CONST_FRA1(a_edgeData[a_dir],a_edgeComp),
		  CHF_FRA1(a_cellData, a_cellComp),
		  CHF_BOX(cellBox),
		  CHF_CONST_INT(a_dir));
  
}





